# -*- coding: utf-8 -*-
"""Bowtie abstract component classes.

All visual and control components inherit these.
"""

# need this for get commands on python2
from __future__ import unicode_literals

# pylint: disable=redefined-builtin
from builtins import bytes

import traceback
import ast
import string
import inspect
from functools import wraps
import json
from datetime import datetime, date, time

import astor
import msgpack
import flask
from flask_socketio import emit
from future.utils import with_metaclass
import eventlet
from eventlet.queue import LightQueue

from bowtie.exceptions import SerializationError


SEPARATOR = '#'


def varname(variable):
    """Return the name of the given variable."""
    # here there be dragons...starting to feel a little uncomfortable
    stack = traceback.extract_stack()
    code = stack[-3][-1]
    tree = ast.parse(code)

    frame = inspect.stack()[2][0]
    for node in ast.walk(tree):
        if hasattr(node, 'attr') and node.attr.startswith('on_'):
            name = astor.to_source(node.value).strip()
            # pylint: disable=eval-used
            if variable is eval(name, frame.f_globals, frame.f_locals):
                return name
    raise Exception('Could not identify name of variable: {}'.format(variable))


def jsbool(x):
    """Convert Python bool to Javascript bool."""
    return repr(x).lower()


def json_conversion(obj):
    """Encode additional objects to JSON."""
    try:
        # numpy isn't an explicit dependency of bowtie
        # so we can't assume it's available
        import numpy as np
        if isinstance(obj, (np.ndarray, np.generic)):
            return obj.tolist()
    except ImportError:
        pass

    try:
        # pandas isn't an explicit dependency of bowtie
        # so we can't assume it's available
        import pandas as pd
        if isinstance(obj, pd.DatetimeIndex):
            return [x.isoformat() for x in obj.to_pydatetime()]
        if isinstance(obj, pd.Index):
            return obj.tolist()
        if isinstance(obj, pd.Series):
            try:
                return [x.isoformat() for x in obj.dt.to_pydatetime()]
            except AttributeError:
                return obj.tolist()
    except ImportError:
        pass

    if isinstance(obj, (datetime, time, date)):
        return obj.isoformat()
    raise TypeError('Not sure how to serialize {} of type {}'.format(obj, type(obj)))


def jdumps(data):
    """Encode Python object to JSON with additional encoders."""
    return json.dumps(data, default=json_conversion)


# pylint: disable=too-many-return-statements
def encoders(obj):
    """Convert Python object to msgpack encodable ones."""
    try:
        # numpy isn't an explicit dependency of bowtie
        # so we can't assume it's available
        import numpy as np
        if isinstance(obj, (np.ndarray, np.generic)):
            # https://docs.scipy.org/doc/numpy/reference/arrays.scalars.html
            return obj.tolist()
    except ImportError:
        pass

    try:
        # pandas isn't an explicit dependency of bowtie
        # so we can't assume it's available
        import pandas as pd
        if isinstance(obj, pd.DatetimeIndex):
            return [x.isoformat() for x in obj.to_pydatetime()]
        if isinstance(obj, pd.Index):
            return obj.tolist()
        if isinstance(obj, pd.Series):
            try:
                return [x.isoformat() for x in obj.dt.to_pydatetime()]
            except AttributeError:
                return obj.tolist()
    except ImportError:
        pass

    if isinstance(obj, (datetime, time, date)):
        return obj.isoformat()

    return obj


def pack(x):
    """Encode ``x`` into msgpack with additional encoders."""
    try:
        return bytes(msgpack.packb(x, default=encoders))
    except TypeError as exc:
        message = ('Serialization error, check the data passed to a do_ command. '
                   'Cannot serialize this object:\n') + str(exc)[16:]
        raise SerializationError(message)


def unpack(x):
    """Decode ``x`` from msgpack into Python object."""
    return msgpack.unpackb(bytes(x['data']), encoding='utf8')


def make_event(event):
    """Create an event from a method signature."""
    # pylint: disable=missing-docstring
    @property
    @wraps(event)
    def actualevent(self):
        name = event.__name__[3:]
        # pylint: disable=protected-access
        ename = '{uuid}{sep}{event}'.format(
            uuid=self._uuid,
            sep=SEPARATOR,
            event=name
        )
        objname = varname(self)
        try:
            # the getter post processing function
            # is preserved with an underscore
            getter = event(self).__name__
        except AttributeError:
            getter = None
        return ename, objname, getter

    return actualevent


def is_event(attribute):
    """Test if a method is an event."""
    return attribute.startswith('on_')


def make_command(command):
    """Create an command from a method signature."""
    # pylint: disable=missing-docstring
    @wraps(command)
    def actualcommand(self, *args, **kwds):
        data = command(self, *args, **kwds)
        name = command.__name__[3:]
        # pylint: disable=protected-access
        signal = '{uuid}{sep}{event}'.format(
            uuid=self._uuid,
            sep=SEPARATOR,
            event=name
        )
        if flask.has_request_context():
            emit(signal, {'data': pack(data)})
        else:
            sio = flask.current_app.extensions['socketio']
            sio.emit(signal, {'data': pack(data)})
        eventlet.sleep()

    return actualcommand


def is_command(attribute):
    """Test if a method is an command."""
    return attribute.startswith('do_')


def make_getter(getter):
    """Create an command from a method signature."""
    # pylint: disable=missing-docstring
    def get(self, timeout=10):
        name = getter.__name__
        # pylint: disable=protected-access
        signal = '{uuid}{sep}{event}'.format(
            uuid=self._uuid,
            sep=SEPARATOR,
            event=name
        )
        event = LightQueue(1)
        if flask.has_request_context():
            emit(signal, callback=lambda x: event.put(unpack(x)))
        else:
            sio = flask.current_app.extensions['socketio']
            sio.emit(signal, callback=lambda x: event.put(unpack(x)))
        data = event.get(timeout=timeout)
        return getter(self, data)

    # don't want to copy the signature in this case
    get.__doc__ = getter.__doc__

    return get


def is_getter(attribute):
    """Test if a method is a getter.

    It can be `get` or `get_*`.
    """
    return attribute.startswith('get')


class _Maker(type):
    def __new__(mcs, name, parents, dct):
        for k in list(dct.keys()):
            if is_event(k):
                dct[k] = make_event(dct[k])
            if is_command(k):
                dct[k] = make_command(dct[k])
            if is_getter(k):
                # preserve the post-processor with an underscore
                dct['_' + k] = dct[k]
                dct[k] = make_getter(dct[k])
        return super(_Maker, mcs).__new__(mcs, name, parents, dct)


class FormatDict(dict):
    """Dict to replace missing keys."""

    def __missing__(self, key):
        """Replace missing key with "{key"}"."""
        return "{" + key + "}"


# pylint: disable=too-few-public-methods
class Component(with_metaclass(_Maker, object)):
    """Abstract class for all components.

    All visual and control classes subclass this so their events
    and commands get transformed by the metaclass.
    """

    _NEXT_UUID = 0

    @classmethod
    def _next_uuid(cls):
        cls._NEXT_UUID += 1
        return cls._NEXT_UUID

    def __init__(self):
        """Give the component a unique ID."""
        # wanted to put "self" instead of "Component"
        # was surprised that didn't work
        self._uuid = Component._next_uuid()
        super(Component, self).__init__()
        self._tagbase = " socket={{socket}} uuid={{'{uuid}'}} />".format(uuid=self._uuid)
        self._tag = '<' + self._COMPONENT
        if self._ATTRS:
            self._tag += ' ' + self._ATTRS

    @staticmethod
    def _insert(wrap, tag):
        """Insert the component tag into the wrapper html.

        This ignores other tags already created like ``{socket}``.

        https://stackoverflow.com/a/11284026/744520
        """
        formatter = string.Formatter()
        mapping = FormatDict(component=tag)
        return formatter.vformat(wrap, (), mapping)



# -*- coding: utf-8 -*-
"""Bowtie pager."""

import flask
from flask_socketio import emit
import eventlet


class Pager(object):
    """Tell the client to send a message to the server."""

    _NEXT_UUID = 0

    @classmethod
    def _next_uuid(cls):
        cls._NEXT_UUID += 1
        return cls._NEXT_UUID

    def __init__(self):
        """Create a pager."""
        self._uuid = Pager._next_uuid()

    def notify(self):
        """Notify the client.

        The function passed to ``App.respond`` will get called.
        """
        if flask.has_request_context():
            emit('page#' + str(self._uuid))
        else:
            sio = flask.current_app.extensions['socketio']
            sio.emit('page#' + str(self._uuid))
        eventlet.sleep()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Test all components for instatiation issues."""

import os
from os import environ as env
import subprocess
import time

import pytest
from bowtie import App
from bowtie import control, visual
from bowtie.tests.utils import reset_uuid

reset_uuid()

# pylint: disable=invalid-name,protected-access
controllers = [getattr(control, comp)() for comp in dir(control)
               if comp[0].isupper() and issubclass(getattr(control, comp), control._Controller) and
               comp != 'Upload']

visuals = [getattr(visual, comp)() for comp in dir(visual)
           if comp[0].isupper() and issubclass(getattr(visual, comp), visual._Visual)]


@pytest.fixture
def components(build_path, monkeypatch):
    """App with all components."""
    monkeypatch.setattr(App, '_sourcefile', lambda self: 'bowtie.tests.test_components')

    app = App(rows=len(visuals))
    for controller in controllers:
        app.add_sidebar(controller)

    for vis in visuals:
        app.add(vis)
    # pylint: disable=protected-access
    app._build()

    env['PYTHONPATH'] = '{}:{}'.format(os.getcwd(), os.environ.get('PYTHONPATH', ''))
    server = subprocess.Popen(os.path.join(build_path, 'src/server.py'), env=env)

    time.sleep(5)
    yield
    server.kill()


# pylint: disable=redefined-outer-name,unused-argument
def test_components(components, chrome_driver):
    """Test that no components cause an error."""
    chrome_driver.get('http://localhost:9991')
    chrome_driver.implicitly_wait(5)

    logs = chrome_driver.get_log('browser')
    for log in logs:
        if log['level'] == 'SEVERE':
            raise Exception(log['message'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Compat testing."""

from bowtie._utils import func_name


def hello():
    """A function."""
    return func_name()


def test_function_names():
    """Test we get the correct function name."""
    assert hello() == 'hello'



# -*- coding: utf-8 -*-
"""Utility functions for testing only."""

from bowtie._component import Component


def reset_uuid():
    """Reset the uuid counter for components."""
    # pylint: disable=protected-access
    Component._NEXT_UUID = 0



# -*- coding: utf-8 -*-
"""Python 2/3 compatability."""

import inspect
import sys
from os import makedirs

IS_PY2 = sys.version_info < (3, 0)

if IS_PY2:
    # pylint: disable=invalid-name
    makedirs_lib = makedirs

    # pylint: disable=function-redefined,missing-docstring
    def makedirs(name, mode=0o777, exist_ok=False):
        """Create directories recursively."""
        try:
            makedirs_lib(name, mode=mode)
        except OSError:
            if not exist_ok:
                raise


def numargs(func):
    """Get number of arguments in Python 3."""
    return len(inspect.signature(func).parameters)


if IS_PY2:
    # pylint: disable=function-redefined
    def numargs(func):
        """Get number of arguments in Python 2."""
        count = 0
        # pylint: disable=deprecated-method
        for args in inspect.getargspec(func)[:2]:
            try:
                count += len(args)
            except TypeError:
                pass
        return count



# -*- coding: utf-8 -*-
"""Test layout functionality."""
# pylint: disable=redefined-outer-name

import pytest

from bowtie import App
from bowtie.control import Button
from bowtie._app import GridIndexError, UsedCellsError, NoUnusedCellsError


@pytest.fixture(scope='module')
def buttons():
    """Four buttons."""
    return [Button() for _ in range(4)]


def app():
    """Simple app."""
    return App(rows=2, columns=2)


def test_all_used(buttons):
    """Test all cells are used."""

    app = App(rows=2, columns=2)
    for i in range(4):
        app.add(buttons[i])

    assert list(app.root.used.values()) == 4 * [True]

    app = App(rows=2, columns=2)
    app[0, 0] = buttons[0]
    app[0, 1] = buttons[1]
    app[1, 0] = buttons[2]
    app[1, 1] = buttons[3]

    assert list(app.root.used.values()) == 4 * [True]

    with pytest.raises(NoUnusedCellsError):
        app.add(buttons[2])

    app = App(rows=2, columns=2)
    app[0] = buttons[0]
    app[1, 0] = buttons[2]
    app[1, 1] = buttons[3]

    assert list(app.root.used.values()) == 4 * [True]

    with pytest.raises(NoUnusedCellsError):
        app.add(buttons[2])


def test_used(buttons):
    """Test cell usage checks."""

    app = App(rows=2, columns=2)
    for i in range(3):
        app.add(buttons[i])

    with pytest.raises(UsedCellsError):
        app[0, 0] = buttons[3]

    with pytest.raises(UsedCellsError):
        app[0:1, 1] = buttons[3]

    with pytest.raises(UsedCellsError):
        app[1, 0:1] = buttons[3]

    app[1, 1] = buttons[3]


def test_grid_index(buttons):
    """Test grid indexing checks."""

    app = App(rows=2, columns=2)
    with pytest.raises(GridIndexError):
        app[-5] = buttons[0]

    app[-1] = buttons[0]

    with pytest.raises(GridIndexError):
        app[2] = buttons[0]

    with pytest.raises(UsedCellsError):
        app[1] = buttons[0]


def test_getitem(buttons):
    """Test grid indexing checks."""

    but = buttons[0]

    app = App(rows=2, columns=2)

    with pytest.raises(GridIndexError):
        app[3] = but

    with pytest.raises(GridIndexError):
        app[1, 2, 3] = but

    with pytest.raises(GridIndexError):
        # pylint: disable=invalid-slice-index
        app['a':3] = but

    with pytest.raises(GridIndexError):
        app['a'] = but

    with pytest.raises(GridIndexError):
        app[3, 'a'] = but

    with pytest.raises(GridIndexError):
        app['a', 3] = but

    with pytest.raises(GridIndexError):
        app[0, 0::2] = but

    with pytest.raises(GridIndexError):
        app[0, 1:-1:-1] = but

    app[1, ] = but
    assert sum(app.root.used.values()) == 2
    app[0, :] = but
    assert sum(app.root.used.values()) == 4

    app = App(rows=2, columns=2)
    app[0:1, 1:2] = but
    assert sum(app.root.used.values()) == 1
    app[1:, 0:] = but
    assert sum(app.root.used.values()) == 3

    app = App(rows=2, columns=2)
    app[-1, :2] = but
    assert sum(app.root.used.values()) == 2

    app = App(rows=1, columns=2)
    app[0, :2] = but
    assert sum(app.root.used.values()) == 2

    app = App(rows=1, columns=2)
    app[0] = but
    assert sum(app.root.used.values()) == 2

    app = App(rows=2, columns=2)
    app[:2] = but
    assert sum(app.root.used.values()) == 4



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Plotly testing."""

import os
from os import environ as env
import subprocess
import time

import pytest
from plotly.graph_objs import Scatter
from plotly.graph_objs import Layout as PlotLayout

from bowtie import App
from bowtie.control import Nouislider, Button
from bowtie.visual import Plotly
from bowtie.tests.utils import reset_uuid


reset_uuid()

# pylint: disable=invalid-name
viz = Plotly()
ctrl = Nouislider()
ctrl2 = Button()


def callback(*args):
    """dummy function"""
    # pylint: disable=unused-argument
    chart = {
        "data": [
            Scatter(x=[1, 2, 3, 4], y=[4, 1, 3, 7])
        ],
        "app": PlotLayout(
            title="hello world"
        )
    }
    viz.do_all(chart)


# pylint: disable=unused-argument
@pytest.fixture
def plotly(build_path, monkeypatch):
    """Create plotly app."""
    monkeypatch.setattr(App, '_sourcefile', lambda self: 'bowtie.tests.test_plotly')

    app = App()
    app.add(viz)
    app.add_sidebar(ctrl)
    app.add_sidebar(ctrl2)
    app.subscribe(callback, ctrl.on_change)
    app.subscribe(callback, ctrl2.on_click)
    # pylint: disable=protected-access
    app._build()

    env['PYTHONPATH'] = '{}:{}'.format(os.getcwd(), os.environ.get('PYTHONPATH', ''))
    server = subprocess.Popen(os.path.join(build_path, 'src/server.py'), env=env)

    time.sleep(5)
    yield
    server.kill()


# pylint: disable=redefined-outer-name,unused-argument
def test_plotly(plotly, chrome_driver):
    """Test plotly component."""
    chrome_driver.get('http://localhost:9991')
    chrome_driver.implicitly_wait(5)

    assert chrome_driver.title == 'Bowtie App'

    button = chrome_driver.find_element_by_class_name('ant-btn')
    button.click()

    points = chrome_driver.find_elements_by_class_name('point')

    logs = chrome_driver.get_log('browser')
    for log in logs:
        if log['level'] == 'SEVERE':
            raise Exception(log['message'])

    assert len(points) == 4



# -*- coding: utf-8 -*-
"""Bowtie cache functions."""

import flask
from flask_socketio import emit
import eventlet
from eventlet.queue import LightQueue
import msgpack

from bowtie._component import pack


def save(key, value):
    """Store the key value pair.

    Parameters
    ----------
    key : str
        The key to determine where it's stored, you'll need this to load the value later.
    value : object
        The value to store in the cache.

    Returns
    -------
    None

    """
    signal = 'cache_save'
    if flask.has_request_context():
        emit(signal, {'key': pack(key), 'data': pack(value)})
    else:
        sio = flask.current_app.extensions['socketio']
        sio.emit(signal, {'key': pack(key), 'data': pack(value)})
    eventlet.sleep()


def load(key):
    """Load the value stored with the key.

    Parameters
    ----------
    key : str
        The key to lookup the value stored.

    Returns
    -------
    object
        The value if the key exists in the cache, otherwise None.

    """
    signal = 'cache_load'
    event = LightQueue(1)
    if flask.has_request_context():
        emit(signal, {'data': pack(key)}, callback=event.put)
    else:
        sio = flask.current_app.extensions['socketio']
        sio.emit(signal, {'data': pack(key)}, callback=event.put)
    return msgpack.unpackb(bytes(event.get(timeout=10)), encoding='utf8')



# -*- coding: utf-8 -*-
"""Defines the App class."""

from __future__ import print_function

import os
import json
from itertools import product
import inspect
import shutil
import stat
from collections import namedtuple, defaultdict, OrderedDict
from subprocess import Popen
import warnings

from jinja2 import Environment, FileSystemLoader

from bowtie._compat import makedirs
from bowtie._component import Component, SEPARATOR
from bowtie.exceptions import (
    GridIndexError, MissingRowOrColumn,
    NoSidebarError, NotStatefulEvent,
    NoUnusedCellsError, SizeError, UsedCellsError,
    WebpackError, YarnError
)


_Import = namedtuple('_Import', ['module', 'component'])
_Control = namedtuple('_Control', ['instantiate', 'caption'])
_Schedule = namedtuple('_Schedule', ['seconds', 'function'])

_DIRECTORY = 'build'
_WEBPACK = './node_modules/.bin/webpack'


def raise_not_number(x):
    """Raise ``SizeError`` if ``x`` is not a number``."""
    try:
        float(x)
    except ValueError:
        raise SizeError('Must pass a number, received {}'.format(x))


class Span(object):
    """Define the location of a widget."""

    # pylint: disable=too-few-public-methods
    def __init__(self, row_start, column_start, row_end=None, column_end=None):
        """Create a span for a widget.

        Indexing starts at 0. Both start and end are inclusive.

        Parameters
        ----------
        row_start : int
        column_start : int
        row_end : int, optional
        column_end : int, optional

        """
        self.row_start = row_start + 1
        self.column_start = column_start + 1
        # add 1 to then ends because they start counting from 1
        if row_end is None:
            self.row_end = self.row_start + 1
        else:
            self.row_end = row_end + 1
        if column_end is None:
            self.column_end = self.column_start + 1
        else:
            self.column_end = column_end + 1

    def __repr__(self):
        """Show the starting and ending points."""
        return '({}, {}) to ({}, {})'.format(
            self.row_start,
            self.column_start,
            self.row_end,
            self.column_end
        )


class Size(object):
    """Size of rows and columns in grid.

    This is accessed through ``.rows`` and ``.columns`` from App and View instances.

    This uses CSS's minmax function.

    The minmax() CSS function defines a size range greater than or equal
    to min and less than or equal to max. If max < min, then max is ignored
    and minmax(min,max) is treated as min. As a maximum, a <flex> value
    sets the flex factor of a grid track; it is invalid as a minimum.

    Examples
    --------
    Laying out an app with the first row using 1/3 of the space
    and the second row using 2/3 of the space.

    >>> app = App(rows=2, columns=3)
    >>> app.rows[0].fraction(1)
    >>> app.rows[1].fraction(2)

    """

    def __init__(self):
        """Create a default row or column size with fraction = 1."""
        self.minimum = None
        self.maximum = None
        self.fraction(1)

    def auto(self):
        """Set the size to auto or content based."""
        self.maximum = 'auto'

    def min_auto(self):
        """Set the minimum size to auto or content based."""
        self.minimum = 'auto'

    def pixels(self, value):
        """Set the size in pixels."""
        raise_not_number(value)
        self.maximum = '{}px'.format(value)

    def min_pixels(self, value):
        """Set the minimum size in pixels."""
        raise_not_number(value)
        self.minimum = '{}px'.format(value)

    def fraction(self, value):
        """Set the fraction of free space to use as an integer."""
        raise_not_number(value)
        self.maximum = '{}fr'.format(int(value))

    def percent(self, value):
        """Set the percentage of free space to use."""
        raise_not_number(value)
        self.maximum = '{}%'.format(value)

    def min_percent(self, value):
        """Set the minimum percentage of free space to use."""
        raise_not_number(value)
        self.minimum = '{}%'.format(value)

    def __repr__(self):
        """Represent the size to be inserted into a JSX template."""
        if self.minimum:
            return 'minmax({}, {})'.format(self.minimum, self.maximum)
        return self.maximum


class Gap(object):
    """Margin between rows or columns of the grid.

    This is accessed through ``.row_gap`` and ``.column_gap`` from App and View instances.

    Examples
    --------
    Create a gap of 5 pixels between all rows.

    >>> app = App()
    >>> app.row_gap.pixels(5)

    """

    def __init__(self):
        """Create a default margin of zero."""
        self.gap = None
        self.pixels(0)

    def pixels(self, value):
        """Set the margin in pixels."""
        raise_not_number(value)
        self.gap = '{}px'.format(value)

    def percent(self, value):
        """Set the margin as a percentage."""
        raise_not_number(value)
        self.gap = '{}%'.format(value)

    def __repr__(self):
        """Represent the margin to be inserted into a JSX template."""
        return self.gap


def _check_index(value, length, bound):
    if value is not None:
        if not isinstance(value, int):
            raise GridIndexError('Indices must be integers, found {}.'.format(value))
        if value < 0:
            value = value + length
        if value < 0 + bound or value >= length + bound:
            raise GridIndexError('Index out of range.')
    return value


def _slice_to_start_end(slc, length):
    if slc.step is not None and slc.step != 1:
        raise GridIndexError(
            'slice step is not supported must be None or 1, was {}'.format(slc.step)
        )

    start = 0
    if slc.start is not None:
        start = slc.start

    end = length
    if slc.stop is not None:
        end = slc.stop
    return start, end


class View(object):
    """Grid of widgets."""

    _NEXT_UUID = 0

    @classmethod
    def _next_uuid(cls):
        cls._NEXT_UUID += 1
        return cls._NEXT_UUID

    def __init__(self, rows=1, columns=1, sidebar=True,
                 background_color='White'):
        """Create a new grid."""
        self._uuid = View._next_uuid()
        self.used = OrderedDict(((key, False) for key in product(range(rows), range(columns))))
        self.column_gap = Gap()
        self.row_gap = Gap()
        self.rows = [Size() for _ in range(rows)]
        self.columns = [Size() for _ in range(columns)]
        self.sidebar = sidebar
        self.background_color = background_color
        self.packages = set()
        self.templates = set()
        self.imports = set()
        self.controllers = []
        self.widgets = []
        self.spans = []

    @property
    def _name(self):
        return 'view{}.jsx'.format(self._uuid)

    def __getitem__(self, key):
        """Get item from the view."""
        raise NotImplementedError('Accessor is not implemented.')

    def __setitem__(self, key, widget):
        """Add widget to the view."""
        if isinstance(key, tuple):
            if len(key) == 1:
                self[key[0]] = widget
            else:
                try:
                    row_key, column_key = key
                except ValueError:
                    raise GridIndexError('Index must be 1 or 2 values, found {}'.format(key))
                if isinstance(row_key, int):
                    row_start = row_key
                    row_end = None
                elif isinstance(row_key, slice):
                    row_start, row_end = _slice_to_start_end(row_key, len(self.rows))
                else:
                    raise GridIndexError(
                        'Cannot index with {}, pass in a int or a slice.'.format(row_key)
                    )

                if isinstance(column_key, int):
                    column_start = column_key
                    column_end = None
                elif isinstance(column_key, slice):
                    column_start, column_end = _slice_to_start_end(column_key, len(self.columns))
                else:
                    raise GridIndexError(
                        'Cannot index with {}, pass in a int or a slice.'.format(column_key)
                    )
                self._add(
                    widget, row_start=row_start, column_start=column_start,
                    row_end=row_end, column_end=column_end
                )

        elif isinstance(key, slice):
            start, end = _slice_to_start_end(key, len(self.rows))
            self._add(
                widget, row_start=start, column_start=0,
                row_end=end, column_end=len(self.columns)
            )
        elif isinstance(key, int):
            self._add(widget, row_start=key, column_start=0, column_end=len(self.columns))
        else:
            raise GridIndexError('Invalid index {}'.format(key))

    def add(self, widget):
        """Add a widget to the grid in the next available cell.

        Searches over columns then rows for available cells.

        Parameters
        ----------
        widget : bowtie._Component
            A Bowtie widget instance.

        """
        self._add(widget)

    def _add(self, widget, row_start=None, column_start=None,
             row_end=None, column_end=None):
        """Add a widget to the grid.

        Zero-based index and exclusive.

        Parameters
        ----------
        widget : bowtie._Component
            A Bowtie widget instance.
        row_start : int, optional
            Starting row for the widget.
        column_start : int, optional
            Starting column for the widget.
        row_end : int, optional
            Ending row for the widget.
        column_end : int, optional
            Ending column for the widget.

        """
        assert isinstance(widget, Component)

        row_start = _check_index(row_start, len(self.rows), False)
        column_start = _check_index(column_start, len(self.columns), False)
        row_end = _check_index(row_end, len(self.rows), True)
        column_end = _check_index(column_end, len(self.columns), True)

        if row_start is not None and row_end is not None and row_start >= row_end:
            raise GridIndexError('row_start: {} must be less than row_end: {}'
                                 .format(row_start, row_end))
        if column_start is not None and column_end is not None and column_start >= column_end:
            raise GridIndexError('column_start: {} must be less than column_end: {}'
                                 .format(column_start, column_end))

        # pylint: disable=protected-access
        self.packages.add(widget._PACKAGE)
        self.templates.add(widget._TEMPLATE)
        self.imports.add(_Import(component=widget._COMPONENT,
                                 module=widget._TEMPLATE[:widget._TEMPLATE.find('.')]))

        if row_start is None or column_start is None:
            if row_start is not None:
                raise MissingRowOrColumn(
                    'Only row_start was defined. '
                    'Please specify both column_start and row_start or neither.'
                )
            if column_start is not None:
                raise MissingRowOrColumn(
                    'Only column_start was defined. '
                    'Please specify both column_start and row_start or neither.'
                )
            if row_end is not None or column_end is not None:
                raise MissingRowOrColumn(
                    'If you specify an end index you must '
                    'specify both row_start and column_start.'
                )
            row, col = None, None
            for (row, col), use in self.used.items():
                if not use:
                    break
            else:
                raise NoUnusedCellsError()
            span = Span(row, col)
            self.used[row, col] = True
        elif row_end is None and column_end is None:
            if self.used[row_start, column_start]:
                raise UsedCellsError('Cell at {}, {} is already used.'
                                     .format(row_start, column_start))
            span = Span(row_start, column_start)
            self.used[row_start, column_start] = True
        else:
            if row_end is None:
                row_end = row_start + 1
            if column_end is None:
                column_end = column_start + 1

            for row, col in product(range(row_start, row_end),
                                    range(column_start, column_end)):
                if self.used[row, col]:
                    raise UsedCellsError('Cell at {}, {} is already used.'.format(row, col))

            for row, col in product(range(row_start, row_end),
                                    range(column_start, column_end)):
                self.used[row, col] = True
            span = Span(row_start, column_start, row_end, column_end)

        self.widgets.append(widget)
        self.spans.append(span)

    def add_sidebar(self, widget):
        """Add a widget to the sidebar.

        Parameters
        ----------
        widget : bowtie._Component
            Add this widget to the sidebar, it will be appended to the end.

        """
        if not self.sidebar:
            raise NoSidebarError('Set `sidebar=True` if you want to use the sidebar.')

        assert isinstance(widget, Component)

        # pylint: disable=protected-access
        self.packages.add(widget._PACKAGE)
        self.templates.add(widget._TEMPLATE)
        self.imports.add(_Import(component=widget._COMPONENT,
                                 module=widget._TEMPLATE[:widget._TEMPLATE.find('.')]))
        self.controllers.append(_Control(instantiate=widget._instantiate,
                                         caption=getattr(widget, 'caption', None)))

    def _render(self, path, env):
        """TODO: Docstring for _render.

        Parameters
        ----------
        path : TODO

        Returns
        -------
        TODO

        """
        jsx = env.get_template('view.jsx.j2')

        # pylint: disable=protected-access
        self.widgets = [w._instantiate for w in self.widgets]

        columns = []
        if self.sidebar:
            columns.append('18em')
        columns += self.columns

        with open(os.path.join(path, self._name), 'w') as f:
            f.write(
                jsx.render(
                    uuid=self._uuid,
                    sidebar=self.sidebar,
                    columns=columns,
                    rows=self.rows,
                    column_gap=self.column_gap,
                    row_gap=self.row_gap,
                    background_color=self.background_color,
                    components=self.imports,
                    controls=self.controllers,
                    widgets=zip(self.widgets, self.spans)
                )
            )


Route = namedtuple('Route', ['view', 'path', 'exact'])


class App(object):
    """Core class to layout, connect, build a Bowtie app."""

    def __init__(self, rows=1, columns=1, sidebar=True,
                 title='Bowtie App', basic_auth=False,
                 username='username', password='password',
                 background_color='White', host='0.0.0.0', port=9991,
                 socketio='', debug=False):
        """Create a Bowtie App.

        Parameters
        ----------
        row : int, optional
            Number of rows in the grid.
        columns : int, optional
            Number of columns in the grid.
        sidebar : bool, optional
            Enable a sidebar for control widgets.
        title : str, optional
            Title of the HTML.
        basic_auth : bool, optional
            Enable basic authentication.
        username : str, optional
            Username for basic authentication.
        password : str, optional
            Password for basic authentication.
        background_color : str, optional
            Background color of the control pane.
        host : str, optional
            Host IP address.
        port : int, optional
            Host port number.
        socketio : string, optional
            Socket.io path prefix, only change this for advanced deployments.
        debug : bool, optional
            Enable debugging in Flask. Disable in production!

        """
        self.background_color = background_color
        self.basic_auth = basic_auth
        self.debug = debug
        self.functions = []
        self.host = host
        self.imports = set()
        self.init = None
        self.password = password
        self.port = port
        self.socketio = socketio
        self.schedules = []
        self.subscriptions = defaultdict(list)
        self.pages = {}
        self.title = title
        self.username = username
        self.uploads = {}
        self.root = View(rows=rows, columns=columns, sidebar=sidebar,
                         background_color=background_color)
        self.routes = [Route(view=self.root, path='/', exact=True)]
        self._package_dir = os.path.dirname(__file__)
        self._jinjaenv = Environment(
            loader=FileSystemLoader(os.path.join(self._package_dir, 'templates')),
            trim_blocks=True,
            lstrip_blocks=True
        )

    def __getattr__(self, name):
        """Export attributes from root view."""
        if name == 'columns':
            return self.root.columns
        elif name == 'rows':
            return self.root.rows
        elif name == 'column_gap':
            return self.root.column_gap
        elif name == 'row_gap':
            return self.root.row_gap
        else:
            raise AttributeError(name)

    def __getitem__(self, key):
        """Get item from root view."""
        self.root.__getitem__(key)

    def __setitem__(self, key, value):
        """Add widget to the root view."""
        self.root.__setitem__(key, value)

    def add(self, widget):
        """Add a widget to the grid in the next available cell.

        Searches over columns then rows for available cells.

        Parameters
        ----------
        widget : bowtie._Component
            A Bowtie widget instance.

        """
        # pylint: disable=protected-access
        self.root._add(widget)

    def add_sidebar(self, widget):
        """Add a widget to the sidebar.

        Parameters
        ----------
        widget : bowtie._Component
            Add this widget to the sidebar, it will be appended to the end.

        """
        self.root.add_sidebar(widget)

    def add_route(self, view, path, exact=True):
        """Add a view to the app.

        Parameters
        ----------
        view : View
        path : str
        exact : bool, optional

        """
        assert path[0] == '/'
        for route in self.routes:
            assert path != route.path, 'Cannot use the same path twice'
        self.routes.append(Route(view=view, path=path, exact=exact))

    def respond(self, pager, func):
        """Call a function in response to a page.

        When the pager calls notify, the function will be called.

        Parameters
        ----------
        pager : Pager
            Pager that to signal when func is called.
        func : callable
            Function to be called.

        Examples
        --------
        Using the pager to run a callback function.

        >>> from bowtie.pager import Pager
        >>> app = App()
        >>> pager = Pager()
        >>> def callback():
        ...     pass
        >>> def scheduledtask():
        ...     pager.notify()
        >>> app.respond(pager, callback)

        """
        self.pages[pager] = func.__name__

    def subscribe(self, func, event, *events):
        """Call a function in response to an event.

        If more than one event is given, `func` will be given
        as many arguments as there are events.

        Parameters
        ----------
        func : callable
            Function to be called.
        event : event
            A Bowtie event.
        *events : Each is an event, optional
            Additional events.

        Examples
        --------
        Subscribing a function to multiple events.

        >>> from bowtie.control import Dropdown, Slider
        >>> app = App()
        >>> dd = Dropdown()
        >>> slide = Slider()
        >>> def callback(dd_item, slide_value):
        ...     pass
        >>> app.subscribe(callback, dd.on_change, slide.on_change)

        """
        all_events = [event]
        all_events.extend(events)

        if len(all_events) > 1:
            # check if we are using any non stateful events
            for evt in all_events:
                if evt[2] is None:
                    name = evt[0].split(SEPARATOR)[1]
                    msg = '{}.on_{} is not a stateful event. It must be used alone.'
                    raise NotStatefulEvent(msg.format(evt[1], name))

        if event[0].split(SEPARATOR)[1] == 'upload':
            # evt = event[0]
            uuid = event[0].split(SEPARATOR)[0]
            if uuid in self.uploads:
                warnings.warn(
                    ('Overwriting function "{func1}" with function '
                     '"{func2}" for upload object "{obj}".').format(
                         func1=self.uploads[uuid],
                         func2=func.__name__,
                         obj=event[1]
                     ), Warning)
            self.uploads[uuid] = func.__name__

        for evt in all_events:
            self.subscriptions[evt].append((all_events, func.__name__))

    def load(self, func):
        """Call a function on page load.

        Parameters
        ----------
        func : callable
            Function to be called.

        """
        self.init = func.__name__

    def schedule(self, seconds, func):
        """Call a function periodically.

        Parameters
        ----------
        seconds : float
            Minimum interval of function calls.
        func : callable
            Function to be called.

        """
        self.schedules.append(_Schedule(seconds, func.__name__))

    # pylint: disable=no-self-use
    def _sourcefile(self):
        # [-1] grabs the top of the stack and [1] grabs the filename
        return os.path.basename(inspect.stack()[-1][1])[:-3]

    def _write_templates(self):
        server = self._jinjaenv.get_template('server.py.j2')
        indexhtml = self._jinjaenv.get_template('index.html.j2')
        indexjsx = self._jinjaenv.get_template('index.jsx.j2')

        src, app, templates = create_directories()

        server_path = os.path.join(src, server.name[:-3])
        with open(server_path, 'w') as f:
            f.write(
                server.render(
                    socketio=self.socketio,
                    basic_auth=self.basic_auth,
                    username=self.username,
                    password=self.password,
                    source_module=self._sourcefile(),
                    subscriptions=self.subscriptions,
                    uploads=self.uploads,
                    schedules=self.schedules,
                    initial=self.init,
                    routes=self.routes,
                    pages=self.pages,
                    host="'{}'".format(self.host),
                    port=self.port,
                    debug=self.debug
                )
            )

        perms = os.stat(server_path)
        os.chmod(server_path, perms.st_mode | stat.S_IEXEC)

        template_src = os.path.join(self._package_dir, 'src', 'progress.jsx')
        shutil.copy(template_src, app)
        template_src = os.path.join(self._package_dir, 'src', 'utils.js')
        shutil.copy(template_src, app)
        for route in self.routes:
            for template in route.view.templates:
                template_src = os.path.join(self._package_dir, 'src', template)
                shutil.copy(template_src, app)

        packages = set()
        for route in self.routes:
            # pylint: disable=protected-access
            route.view._render(app, self._jinjaenv)
            packages |= route.view.packages

        with open(os.path.join(templates, indexhtml.name[:-3]), 'w') as f:
            f.write(
                indexhtml.render(
                    title=self.title,
                )
            )

        with open(os.path.join(app, indexjsx.name[:-3]), 'w') as f:
            f.write(
                indexjsx.render(
                    # pylint: disable=protected-access
                    maxviewid=View._NEXT_UUID,
                    socketio=self.socketio,
                    pages=self.pages,
                    routes=self.routes
                )
            )
        return packages

    def build(self):
        """Deprecate build function."""
        warnings.warn(
            '`build` is deprecated, return the `App` object instead.',
            DeprecationWarning
        )

    def _build(self):
        """Compile the Bowtie application."""
        packages = self._write_templates()

        if not os.path.isfile(os.path.join(_DIRECTORY, 'webpack.config.js')):
            webpack_src = os.path.join(self._package_dir, 'src/webpack.config.js')
            shutil.copy(webpack_src, _DIRECTORY)

        if not os.path.isfile(os.path.join(_DIRECTORY, 'package.json')):
            packagejson = os.path.join(self._package_dir, 'src/package.json')
            shutil.copy(packagejson, _DIRECTORY)

        install = Popen('yarn install', shell=True, cwd=_DIRECTORY).wait()
        if install > 1:
            raise YarnError('Error install node packages')

        packages.discard(None)
        if packages:
            installed = installed_packages()
            packages = [x for x in packages if x.split('@')[0] not in installed]

            if packages:
                packagestr = ' '.join(packages)
                install = Popen('yarn add {}'.format(packagestr),
                                shell=True, cwd=_DIRECTORY).wait()
                if install > 1:
                    raise YarnError('Error install node packages')
                elif install == 1:
                    print('Yarn error but trying to continue build')
        dev = Popen('{} -d'.format(_WEBPACK), shell=True, cwd=_DIRECTORY).wait()
        if dev != 0:
            raise WebpackError('Error building with webpack')


def installed_packages():
    """Extract installed packages as list from `package.json`."""
    with open(os.path.join(_DIRECTORY, 'package.json'), 'r') as f:
        packagejson = json.load(f)
    return packagejson['dependencies'].keys()


def create_directories():
    """Create all the necessary subdirectories for the build."""
    src = os.path.join(_DIRECTORY, 'src')
    templates = os.path.join(src, 'templates')
    app = os.path.join(src, 'app')
    makedirs(app, exist_ok=True)
    makedirs(templates, exist_ok=True)
    return src, app, templates



# -*- coding: utf-8 -*-
"""Test tag instantation for components."""

from bowtie.visual import Markdown


def test_markdown():
    """Test tags for the Markdown widget."""
    # pylint: disable=protected-access
    next_uuid = Markdown._NEXT_UUID
    mark = Markdown()
    assert mark._instantiate == ("<AntProgress socket={{socket}} uuid={{'{next_uuid}'}}>"
                                 "<Markdown initial={{''}} "
                                 "socket={{socket}} uuid={{'{next_uuid1}'}} />"
                                 "</AntProgress>").format(next_uuid=next_uuid + 1,
                                                          next_uuid1=next_uuid + 2)

    next_uuid = Markdown._NEXT_UUID
    mark = Markdown(initial='#hi\n##hello')
    assert mark._instantiate == ("<AntProgress socket={{socket}} uuid={{'{next_uuid}'}}>"
                                 "<Markdown initial={{'<h1>hi</h1>\\n<h2>hello</h2>'}} "
                                 "socket={{socket}} uuid={{'{next_uuid1}'}} />"
                                 "</AntProgress>").format(next_uuid=next_uuid + 1,
                                                          next_uuid1=next_uuid + 2)



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['bowtie', 'bowtie.feedback', 'bowtie.tests']

package_data = \
{'': ['*'],
 'bowtie': ['src/*',
            'src/node_modules/*',
            'src/node_modules/.bin/*',
            'src/node_modules/abs/*',
            'src/node_modules/abs/lib/*',
            'src/node_modules/ansi-regex/*',
            'src/node_modules/ansi-styles/*',
            'src/node_modules/anymatch/*',
            'src/node_modules/arr-diff/*',
            'src/node_modules/arr-flatten/*',
            'src/node_modules/array-unique/*',
            'src/node_modules/async-each/*',
            'src/node_modules/babel-cli/*',
            'src/node_modules/babel-cli/bin/*',
            'src/node_modules/babel-cli/lib/*',
            'src/node_modules/babel-cli/lib/babel/*',
            'src/node_modules/babel-cli/node_modules/*',
            'src/node_modules/babel-cli/node_modules/babel-core/*',
            'src/node_modules/babel-cli/node_modules/babel-core/lib/*',
            'src/node_modules/babel-cli/node_modules/babel-core/lib/api/*',
            'src/node_modules/babel-cli/node_modules/babel-core/lib/helpers/*',
            'src/node_modules/babel-cli/node_modules/babel-core/lib/tools/*',
            'src/node_modules/babel-cli/node_modules/babel-core/lib/transformation/*',
            'src/node_modules/babel-cli/node_modules/babel-core/lib/transformation/file/*',
            'src/node_modules/babel-cli/node_modules/babel-core/lib/transformation/file/options/*',
            'src/node_modules/babel-cli/node_modules/babel-core/lib/transformation/internal-plugins/*',
            'src/node_modules/babel-cli/node_modules/babel-polyfill/*',
            'src/node_modules/babel-cli/node_modules/babel-polyfill/dist/*',
            'src/node_modules/babel-cli/node_modules/babel-polyfill/lib/*',
            'src/node_modules/babel-cli/node_modules/babel-polyfill/scripts/*',
            'src/node_modules/babel-cli/node_modules/core-js/*',
            'src/node_modules/babel-cli/node_modules/core-js/build/*',
            'src/node_modules/babel-cli/node_modules/core-js/client/*',
            'src/node_modules/babel-cli/node_modules/core-js/core/*',
            'src/node_modules/babel-cli/node_modules/core-js/es5/*',
            'src/node_modules/babel-cli/node_modules/core-js/es6/*',
            'src/node_modules/babel-cli/node_modules/core-js/es7/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/array/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/array/virtual/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/date/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/dom-collections/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/error/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/function/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/function/virtual/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/json/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/map/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/math/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/number/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/number/virtual/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/object/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/promise/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/reflect/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/regexp/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/set/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/string/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/string/virtual/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/symbol/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/system/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/typed/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/weak-map/*',
            'src/node_modules/babel-cli/node_modules/core-js/fn/weak-set/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/core/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/es5/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/es6/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/es7/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/array/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/array/virtual/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/date/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/dom-collections/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/error/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/function/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/function/virtual/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/json/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/map/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/math/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/number/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/number/virtual/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/object/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/promise/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/reflect/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/regexp/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/set/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/string/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/string/virtual/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/symbol/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/system/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/typed/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/weak-map/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/fn/weak-set/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/modules/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/stage/*',
            'src/node_modules/babel-cli/node_modules/core-js/library/web/*',
            'src/node_modules/babel-cli/node_modules/core-js/modules/*',
            'src/node_modules/babel-cli/node_modules/core-js/modules/library/*',
            'src/node_modules/babel-cli/node_modules/core-js/stage/*',
            'src/node_modules/babel-cli/node_modules/core-js/web/*',
            'src/node_modules/babel-cli/node_modules/regenerator-runtime/*',
            'src/node_modules/babel-cli/scripts/*',
            'src/node_modules/babel-code-frame/*',
            'src/node_modules/babel-code-frame/lib/*',
            'src/node_modules/babel-generator/*',
            'src/node_modules/babel-generator/lib/*',
            'src/node_modules/babel-generator/lib/generators/*',
            'src/node_modules/babel-generator/lib/node/*',
            'src/node_modules/babel-helper-bindify-decorators/*',
            'src/node_modules/babel-helper-bindify-decorators/lib/*',
            'src/node_modules/babel-helper-builder-binary-assignment-operator-visitor/*',
            'src/node_modules/babel-helper-builder-binary-assignment-operator-visitor/lib/*',
            'src/node_modules/babel-helper-builder-react-jsx/*',
            'src/node_modules/babel-helper-builder-react-jsx/lib/*',
            'src/node_modules/babel-helper-call-delegate/*',
            'src/node_modules/babel-helper-call-delegate/lib/*',
            'src/node_modules/babel-helper-define-map/*',
            'src/node_modules/babel-helper-define-map/lib/*',
            'src/node_modules/babel-helper-explode-assignable-expression/*',
            'src/node_modules/babel-helper-explode-assignable-expression/lib/*',
            'src/node_modules/babel-helper-explode-class/*',
            'src/node_modules/babel-helper-explode-class/lib/*',
            'src/node_modules/babel-helper-function-name/*',
            'src/node_modules/babel-helper-function-name/lib/*',
            'src/node_modules/babel-helper-get-function-arity/*',
            'src/node_modules/babel-helper-get-function-arity/lib/*',
            'src/node_modules/babel-helper-hoist-variables/*',
            'src/node_modules/babel-helper-hoist-variables/lib/*',
            'src/node_modules/babel-helper-optimise-call-expression/*',
            'src/node_modules/babel-helper-optimise-call-expression/lib/*',
            'src/node_modules/babel-helper-regex/*',
            'src/node_modules/babel-helper-regex/lib/*',
            'src/node_modules/babel-helper-remap-async-to-generator/*',
            'src/node_modules/babel-helper-remap-async-to-generator/lib/*',
            'src/node_modules/babel-helper-replace-supers/*',
            'src/node_modules/babel-helper-replace-supers/lib/*',
            'src/node_modules/babel-helpers/*',
            'src/node_modules/babel-helpers/lib/*',
            'src/node_modules/babel-messages/*',
            'src/node_modules/babel-messages/lib/*',
            'src/node_modules/babel-plugin-check-es2015-constants/*',
            'src/node_modules/babel-plugin-check-es2015-constants/lib/*',
            'src/node_modules/babel-plugin-syntax-async-functions/*',
            'src/node_modules/babel-plugin-syntax-async-functions/lib/*',
            'src/node_modules/babel-plugin-syntax-async-generators/*',
            'src/node_modules/babel-plugin-syntax-async-generators/lib/*',
            'src/node_modules/babel-plugin-syntax-class-constructor-call/*',
            'src/node_modules/babel-plugin-syntax-class-constructor-call/lib/*',
            'src/node_modules/babel-plugin-syntax-class-properties/*',
            'src/node_modules/babel-plugin-syntax-class-properties/lib/*',
            'src/node_modules/babel-plugin-syntax-decorators/*',
            'src/node_modules/babel-plugin-syntax-decorators/lib/*',
            'src/node_modules/babel-plugin-syntax-do-expressions/*',
            'src/node_modules/babel-plugin-syntax-do-expressions/lib/*',
            'src/node_modules/babel-plugin-syntax-dynamic-import/*',
            'src/node_modules/babel-plugin-syntax-dynamic-import/lib/*',
            'src/node_modules/babel-plugin-syntax-exponentiation-operator/*',
            'src/node_modules/babel-plugin-syntax-exponentiation-operator/lib/*',
            'src/node_modules/babel-plugin-syntax-export-extensions/*',
            'src/node_modules/babel-plugin-syntax-export-extensions/lib/*',
            'src/node_modules/babel-plugin-syntax-flow/*',
            'src/node_modules/babel-plugin-syntax-flow/lib/*',
            'src/node_modules/babel-plugin-syntax-function-bind/*',
            'src/node_modules/babel-plugin-syntax-function-bind/lib/*',
            'src/node_modules/babel-plugin-syntax-jsx/*',
            'src/node_modules/babel-plugin-syntax-jsx/lib/*',
            'src/node_modules/babel-plugin-syntax-object-rest-spread/*',
            'src/node_modules/babel-plugin-syntax-object-rest-spread/lib/*',
            'src/node_modules/babel-plugin-syntax-trailing-function-commas/*',
            'src/node_modules/babel-plugin-syntax-trailing-function-commas/lib/*',
            'src/node_modules/babel-plugin-transform-async-generator-functions/*',
            'src/node_modules/babel-plugin-transform-async-generator-functions/lib/*',
            'src/node_modules/babel-plugin-transform-async-to-generator/*',
            'src/node_modules/babel-plugin-transform-async-to-generator/lib/*',
            'src/node_modules/babel-plugin-transform-class-constructor-call/*',
            'src/node_modules/babel-plugin-transform-class-constructor-call/lib/*',
            'src/node_modules/babel-plugin-transform-class-properties/*',
            'src/node_modules/babel-plugin-transform-class-properties/lib/*',
            'src/node_modules/babel-plugin-transform-decorators/*',
            'src/node_modules/babel-plugin-transform-decorators/lib/*',
            'src/node_modules/babel-plugin-transform-do-expressions/*',
            'src/node_modules/babel-plugin-transform-do-expressions/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-arrow-functions/*',
            'src/node_modules/babel-plugin-transform-es2015-arrow-functions/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-block-scoped-functions/*',
            'src/node_modules/babel-plugin-transform-es2015-block-scoped-functions/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-block-scoping/*',
            'src/node_modules/babel-plugin-transform-es2015-block-scoping/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-classes/*',
            'src/node_modules/babel-plugin-transform-es2015-classes/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-classes/lib/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-computed-properties/*',
            'src/node_modules/babel-plugin-transform-es2015-computed-properties/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-destructuring/*',
            'src/node_modules/babel-plugin-transform-es2015-destructuring/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-duplicate-keys/*',
            'src/node_modules/babel-plugin-transform-es2015-duplicate-keys/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-for-of/*',
            'src/node_modules/babel-plugin-transform-es2015-for-of/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-function-name/*',
            'src/node_modules/babel-plugin-transform-es2015-function-name/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-literals/*',
            'src/node_modules/babel-plugin-transform-es2015-literals/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-modules-amd/*',
            'src/node_modules/babel-plugin-transform-es2015-modules-amd/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-modules-commonjs/*',
            'src/node_modules/babel-plugin-transform-es2015-modules-commonjs/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-modules-systemjs/*',
            'src/node_modules/babel-plugin-transform-es2015-modules-systemjs/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-modules-umd/*',
            'src/node_modules/babel-plugin-transform-es2015-modules-umd/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-object-super/*',
            'src/node_modules/babel-plugin-transform-es2015-object-super/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-parameters/*',
            'src/node_modules/babel-plugin-transform-es2015-parameters/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-shorthand-properties/*',
            'src/node_modules/babel-plugin-transform-es2015-shorthand-properties/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-spread/*',
            'src/node_modules/babel-plugin-transform-es2015-spread/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-sticky-regex/*',
            'src/node_modules/babel-plugin-transform-es2015-sticky-regex/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-template-literals/*',
            'src/node_modules/babel-plugin-transform-es2015-template-literals/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-typeof-symbol/*',
            'src/node_modules/babel-plugin-transform-es2015-typeof-symbol/lib/*',
            'src/node_modules/babel-plugin-transform-es2015-unicode-regex/*',
            'src/node_modules/babel-plugin-transform-es2015-unicode-regex/lib/*',
            'src/node_modules/babel-plugin-transform-exponentiation-operator/*',
            'src/node_modules/babel-plugin-transform-exponentiation-operator/lib/*',
            'src/node_modules/babel-plugin-transform-export-extensions/*',
            'src/node_modules/babel-plugin-transform-export-extensions/lib/*',
            'src/node_modules/babel-plugin-transform-flow-strip-types/*',
            'src/node_modules/babel-plugin-transform-flow-strip-types/lib/*',
            'src/node_modules/babel-plugin-transform-function-bind/*',
            'src/node_modules/babel-plugin-transform-function-bind/lib/*',
            'src/node_modules/babel-plugin-transform-react-display-name/*',
            'src/node_modules/babel-plugin-transform-react-display-name/lib/*',
            'src/node_modules/babel-plugin-transform-react-jsx-self/*',
            'src/node_modules/babel-plugin-transform-react-jsx-self/lib/*',
            'src/node_modules/babel-plugin-transform-react-jsx-source/*',
            'src/node_modules/babel-plugin-transform-react-jsx-source/lib/*',
            'src/node_modules/babel-plugin-transform-react-jsx/*',
            'src/node_modules/babel-plugin-transform-react-jsx/lib/*',
            'src/node_modules/babel-plugin-transform-regenerator/*',
            'src/node_modules/babel-plugin-transform-regenerator/lib/*',
            'src/node_modules/babel-plugin-transform-strict-mode/*',
            'src/node_modules/babel-plugin-transform-strict-mode/lib/*',
            'src/node_modules/babel-preset-es2015/*',
            'src/node_modules/babel-preset-es2015/lib/*',
            'src/node_modules/babel-preset-flow/*',
            'src/node_modules/babel-preset-flow/lib/*',
            'src/node_modules/babel-preset-react/*',
            'src/node_modules/babel-preset-react/lib/*',
            'src/node_modules/babel-preset-stage-0/*',
            'src/node_modules/babel-preset-stage-0/lib/*',
            'src/node_modules/babel-preset-stage-1/*',
            'src/node_modules/babel-preset-stage-1/lib/*',
            'src/node_modules/babel-preset-stage-2/*',
            'src/node_modules/babel-preset-stage-2/lib/*',
            'src/node_modules/babel-preset-stage-3/*',
            'src/node_modules/babel-preset-stage-3/lib/*',
            'src/node_modules/babel-preset-stage-3/node_modules/*',
            'src/node_modules/babel-preset-stage-3/node_modules/babel-plugin-transform-object-rest-spread/*',
            'src/node_modules/babel-preset-stage-3/node_modules/babel-plugin-transform-object-rest-spread/lib/*',
            'src/node_modules/babel-register/*',
            'src/node_modules/babel-register/lib/*',
            'src/node_modules/babel-register/node_modules/*',
            'src/node_modules/babel-register/node_modules/babel-core/*',
            'src/node_modules/babel-register/node_modules/babel-core/lib/*',
            'src/node_modules/babel-register/node_modules/babel-core/lib/api/*',
            'src/node_modules/babel-register/node_modules/babel-core/lib/helpers/*',
            'src/node_modules/babel-register/node_modules/babel-core/lib/tools/*',
            'src/node_modules/babel-register/node_modules/babel-core/lib/transformation/*',
            'src/node_modules/babel-register/node_modules/babel-core/lib/transformation/file/*',
            'src/node_modules/babel-register/node_modules/babel-core/lib/transformation/file/options/*',
            'src/node_modules/babel-register/node_modules/babel-core/lib/transformation/internal-plugins/*',
            'src/node_modules/babel-register/node_modules/core-js/*',
            'src/node_modules/babel-register/node_modules/core-js/build/*',
            'src/node_modules/babel-register/node_modules/core-js/client/*',
            'src/node_modules/babel-register/node_modules/core-js/core/*',
            'src/node_modules/babel-register/node_modules/core-js/es5/*',
            'src/node_modules/babel-register/node_modules/core-js/es6/*',
            'src/node_modules/babel-register/node_modules/core-js/es7/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/array/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/array/virtual/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/date/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/dom-collections/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/error/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/function/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/function/virtual/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/json/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/map/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/math/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/number/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/number/virtual/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/object/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/promise/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/reflect/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/regexp/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/set/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/string/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/string/virtual/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/symbol/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/system/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/typed/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/weak-map/*',
            'src/node_modules/babel-register/node_modules/core-js/fn/weak-set/*',
            'src/node_modules/babel-register/node_modules/core-js/library/*',
            'src/node_modules/babel-register/node_modules/core-js/library/core/*',
            'src/node_modules/babel-register/node_modules/core-js/library/es5/*',
            'src/node_modules/babel-register/node_modules/core-js/library/es6/*',
            'src/node_modules/babel-register/node_modules/core-js/library/es7/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/array/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/array/virtual/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/date/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/dom-collections/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/error/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/function/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/function/virtual/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/json/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/map/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/math/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/number/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/number/virtual/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/object/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/promise/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/reflect/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/regexp/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/set/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/string/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/string/virtual/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/symbol/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/system/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/typed/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/weak-map/*',
            'src/node_modules/babel-register/node_modules/core-js/library/fn/weak-set/*',
            'src/node_modules/babel-register/node_modules/core-js/library/modules/*',
            'src/node_modules/babel-register/node_modules/core-js/library/stage/*',
            'src/node_modules/babel-register/node_modules/core-js/library/web/*',
            'src/node_modules/babel-register/node_modules/core-js/modules/*',
            'src/node_modules/babel-register/node_modules/core-js/modules/library/*',
            'src/node_modules/babel-register/node_modules/core-js/stage/*',
            'src/node_modules/babel-register/node_modules/core-js/web/*',
            'src/node_modules/babel-runtime/*',
            'src/node_modules/babel-runtime/core-js/*',
            'src/node_modules/babel-runtime/core-js/array/*',
            'src/node_modules/babel-runtime/core-js/error/*',
            'src/node_modules/babel-runtime/core-js/json/*',
            'src/node_modules/babel-runtime/core-js/math/*',
            'src/node_modules/babel-runtime/core-js/number/*',
            'src/node_modules/babel-runtime/core-js/object/*',
            'src/node_modules/babel-runtime/core-js/reflect/*',
            'src/node_modules/babel-runtime/core-js/regexp/*',
            'src/node_modules/babel-runtime/core-js/string/*',
            'src/node_modules/babel-runtime/core-js/symbol/*',
            'src/node_modules/babel-runtime/core-js/system/*',
            'src/node_modules/babel-runtime/helpers/*',
            'src/node_modules/babel-runtime/node_modules/*',
            'src/node_modules/babel-runtime/node_modules/core-js/*',
            'src/node_modules/babel-runtime/node_modules/core-js/build/*',
            'src/node_modules/babel-runtime/node_modules/core-js/client/*',
            'src/node_modules/babel-runtime/node_modules/core-js/core/*',
            'src/node_modules/babel-runtime/node_modules/core-js/es5/*',
            'src/node_modules/babel-runtime/node_modules/core-js/es6/*',
            'src/node_modules/babel-runtime/node_modules/core-js/es7/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/array/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/array/virtual/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/date/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/dom-collections/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/error/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/function/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/function/virtual/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/json/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/map/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/math/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/number/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/number/virtual/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/object/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/promise/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/reflect/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/regexp/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/set/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/string/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/string/virtual/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/symbol/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/system/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/typed/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/weak-map/*',
            'src/node_modules/babel-runtime/node_modules/core-js/fn/weak-set/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/core/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/es5/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/es6/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/es7/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/array/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/array/virtual/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/date/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/dom-collections/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/error/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/function/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/function/virtual/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/json/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/map/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/math/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/number/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/number/virtual/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/object/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/promise/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/reflect/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/regexp/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/set/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/string/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/string/virtual/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/symbol/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/system/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/typed/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/weak-map/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/fn/weak-set/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/modules/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/stage/*',
            'src/node_modules/babel-runtime/node_modules/core-js/library/web/*',
            'src/node_modules/babel-runtime/node_modules/core-js/modules/*',
            'src/node_modules/babel-runtime/node_modules/core-js/modules/library/*',
            'src/node_modules/babel-runtime/node_modules/core-js/stage/*',
            'src/node_modules/babel-runtime/node_modules/core-js/web/*',
            'src/node_modules/babel-runtime/regenerator/*',
            'src/node_modules/babel-template/*',
            'src/node_modules/babel-template/lib/*',
            'src/node_modules/babel-traverse/*',
            'src/node_modules/babel-traverse/lib/*',
            'src/node_modules/babel-traverse/lib/path/*',
            'src/node_modules/babel-traverse/lib/path/inference/*',
            'src/node_modules/babel-traverse/lib/path/lib/*',
            'src/node_modules/babel-traverse/lib/scope/*',
            'src/node_modules/babel-traverse/lib/scope/lib/*',
            'src/node_modules/babel-types/*',
            'src/node_modules/babel-types/lib/*',
            'src/node_modules/babel-types/lib/definitions/*',
            'src/node_modules/babel/*',
            'src/node_modules/babel/lib/*',
            'src/node_modules/babylon/*',
            'src/node_modules/babylon/bin/*',
            'src/node_modules/babylon/lib/*',
            'src/node_modules/balanced-match/*',
            'src/node_modules/binary-extensions/*',
            'src/node_modules/brace-expansion/*',
            'src/node_modules/braces/*',
            'src/node_modules/builtin-modules/*',
            'src/node_modules/capture-stack-trace/*',
            'src/node_modules/chalk/*',
            'src/node_modules/chokidar/*',
            'src/node_modules/chokidar/lib/*',
            'src/node_modules/commander/*',
            'src/node_modules/commander/typings/*',
            'src/node_modules/concat-map/*',
            'src/node_modules/concat-map/example/*',
            'src/node_modules/concat-map/test/*',
            'src/node_modules/convert-source-map/*',
            'src/node_modules/core-util-is/*',
            'src/node_modules/core-util-is/lib/*',
            'src/node_modules/create-error-class/*',
            'src/node_modules/debug/*',
            'src/node_modules/debug/src/*',
            'src/node_modules/deep-extend/*',
            'src/node_modules/deep-extend/lib/*',
            'src/node_modules/deffy/*',
            'src/node_modules/deffy/lib/*',
            'src/node_modules/detect-indent/*',
            'src/node_modules/duplexer2/*',
            'src/node_modules/err/*',
            'src/node_modules/err/example/*',
            'src/node_modules/err/lib/*',
            'src/node_modules/error-ex/*',
            'src/node_modules/escape-string-regexp/*',
            'src/node_modules/esutils/*',
            'src/node_modules/esutils/lib/*',
            'src/node_modules/exec-limiter/*',
            'src/node_modules/exec-limiter/lib/*',
            'src/node_modules/expand-brackets/*',
            'src/node_modules/expand-range/*',
            'src/node_modules/extglob/*',
            'src/node_modules/filename-regex/*',
            'src/node_modules/fill-range/*',
            'src/node_modules/for-in/*',
            'src/node_modules/for-own/*',
            'src/node_modules/fs-readdir-recursive/*',
            'src/node_modules/fs.realpath/*',
            'src/node_modules/fsevents/*',
            'src/node_modules/fsevents/build/*',
            'src/node_modules/fsevents/build/Release/*',
            'src/node_modules/fsevents/build/Release/.deps/*',
            'src/node_modules/fsevents/build/Release/.deps/Release/*',
            'src/node_modules/fsevents/build/Release/.deps/Release/obj.target/*',
            'src/node_modules/fsevents/build/Release/.deps/Release/obj.target/fse/*',
            'src/node_modules/fsevents/build/Release/.deps/Users/*',
            'src/node_modules/fsevents/build/Release/.deps/Users/eshanker/*',
            'src/node_modules/fsevents/build/Release/.deps/Users/eshanker/Code/*',
            'src/node_modules/fsevents/build/Release/.deps/Users/eshanker/Code/fsevents/*',
            'src/node_modules/fsevents/build/Release/.deps/Users/eshanker/Code/fsevents/lib/*',
            'src/node_modules/fsevents/build/Release/.deps/Users/eshanker/Code/fsevents/lib/binding/*',
            'src/node_modules/fsevents/build/Release/.deps/Users/eshanker/Code/fsevents/lib/binding/Release/*',
            'src/node_modules/fsevents/build/Release/.deps/Users/eshanker/Code/fsevents/lib/binding/Release/node-v48-darwin-x64/*',
            'src/node_modules/fsevents/build/Release/obj.target/*',
            'src/node_modules/fsevents/build/Release/obj.target/fse/*',
            'src/node_modules/fsevents/build/stage/*',
            'src/node_modules/fsevents/build/stage/v1.1.2/*',
            'src/node_modules/fsevents/lib/*',
            'src/node_modules/fsevents/lib/binding/*',
            'src/node_modules/fsevents/lib/binding/Release/*',
            'src/node_modules/fsevents/lib/binding/Release/node-v11-darwin-x64/*',
            'src/node_modules/fsevents/lib/binding/Release/node-v46-darwin-x64/*',
            'src/node_modules/fsevents/lib/binding/Release/node-v47-darwin-x64/*',
            'src/node_modules/fsevents/lib/binding/Release/node-v48-darwin-x64/*',
            'src/node_modules/fsevents/lib/binding/Release/node-v57-darwin-x64/*',
            'src/node_modules/fsevents/lib/binding/Release/node-v59-darwin-x64/*',
            'src/node_modules/fsevents/node_modules/*',
            'src/node_modules/fsevents/node_modules/.bin/*',
            'src/node_modules/fsevents/node_modules/abbrev/*',
            'src/node_modules/fsevents/node_modules/ajv/*',
            'src/node_modules/fsevents/node_modules/ajv/dist/*',
            'src/node_modules/fsevents/node_modules/ajv/lib/*',
            'src/node_modules/fsevents/node_modules/ajv/lib/compile/*',
            'src/node_modules/fsevents/node_modules/ajv/lib/dot/*',
            'src/node_modules/fsevents/node_modules/ajv/lib/dot/v5/*',
            'src/node_modules/fsevents/node_modules/ajv/lib/dotjs/*',
            'src/node_modules/fsevents/node_modules/ajv/lib/refs/*',
            'src/node_modules/fsevents/node_modules/ajv/scripts/*',
            'src/node_modules/fsevents/node_modules/ansi-regex/*',
            'src/node_modules/fsevents/node_modules/aproba/*',
            'src/node_modules/fsevents/node_modules/are-we-there-yet/*',
            'src/node_modules/fsevents/node_modules/asn1/*',
            'src/node_modules/fsevents/node_modules/asn1/lib/*',
            'src/node_modules/fsevents/node_modules/asn1/lib/ber/*',
            'src/node_modules/fsevents/node_modules/asn1/tst/*',
            'src/node_modules/fsevents/node_modules/asn1/tst/ber/*',
            'src/node_modules/fsevents/node_modules/assert-plus/*',
            'src/node_modules/fsevents/node_modules/asynckit/*',
            'src/node_modules/fsevents/node_modules/asynckit/lib/*',
            'src/node_modules/fsevents/node_modules/aws-sign2/*',
            'src/node_modules/fsevents/node_modules/aws4/*',
            'src/node_modules/fsevents/node_modules/balanced-match/*',
            'src/node_modules/fsevents/node_modules/bcrypt-pbkdf/*',
            'src/node_modules/fsevents/node_modules/block-stream/*',
            'src/node_modules/fsevents/node_modules/boom/*',
            'src/node_modules/fsevents/node_modules/boom/images/*',
            'src/node_modules/fsevents/node_modules/boom/lib/*',
            'src/node_modules/fsevents/node_modules/boom/test/*',
            'src/node_modules/fsevents/node_modules/brace-expansion/*',
            'src/node_modules/fsevents/node_modules/buffer-shims/*',
            'src/node_modules/fsevents/node_modules/caseless/*',
            'src/node_modules/fsevents/node_modules/co/*',
            'src/node_modules/fsevents/node_modules/code-point-at/*',
            'src/node_modules/fsevents/node_modules/combined-stream/*',
            'src/node_modules/fsevents/node_modules/combined-stream/lib/*',
            'src/node_modules/fsevents/node_modules/concat-map/*',
            'src/node_modules/fsevents/node_modules/concat-map/example/*',
            'src/node_modules/fsevents/node_modules/concat-map/test/*',
            'src/node_modules/fsevents/node_modules/console-control-strings/*',
            'src/node_modules/fsevents/node_modules/core-util-is/*',
            'src/node_modules/fsevents/node_modules/core-util-is/lib/*',
            'src/node_modules/fsevents/node_modules/cryptiles/*',
            'src/node_modules/fsevents/node_modules/cryptiles/lib/*',
            'src/node_modules/fsevents/node_modules/cryptiles/test/*',
            'src/node_modules/fsevents/node_modules/dashdash/*',
            'src/node_modules/fsevents/node_modules/dashdash/etc/*',
            'src/node_modules/fsevents/node_modules/dashdash/lib/*',
            'src/node_modules/fsevents/node_modules/dashdash/node_modules/*',
            'src/node_modules/fsevents/node_modules/dashdash/node_modules/assert-plus/*',
            'src/node_modules/fsevents/node_modules/debug/*',
            'src/node_modules/fsevents/node_modules/debug/src/*',
            'src/node_modules/fsevents/node_modules/deep-extend/*',
            'src/node_modules/fsevents/node_modules/deep-extend/lib/*',
            'src/node_modules/fsevents/node_modules/delayed-stream/*',
            'src/node_modules/fsevents/node_modules/delayed-stream/lib/*',
            'src/node_modules/fsevents/node_modules/delegates/*',
            'src/node_modules/fsevents/node_modules/delegates/test/*',
            'src/node_modules/fsevents/node_modules/detect-libc/*',
            'src/node_modules/fsevents/node_modules/detect-libc/bin/*',
            'src/node_modules/fsevents/node_modules/detect-libc/lib/*',
            'src/node_modules/fsevents/node_modules/ecc-jsbn/*',
            'src/node_modules/fsevents/node_modules/ecc-jsbn/lib/*',
            'src/node_modules/fsevents/node_modules/extend/*',
            'src/node_modules/fsevents/node_modules/extsprintf/*',
            'src/node_modules/fsevents/node_modules/extsprintf/examples/*',
            'src/node_modules/fsevents/node_modules/extsprintf/lib/*',
            'src/node_modules/fsevents/node_modules/forever-agent/*',
            'src/node_modules/fsevents/node_modules/form-data/*',
            'src/node_modules/fsevents/node_modules/form-data/lib/*',
            'src/node_modules/fsevents/node_modules/fs.realpath/*',
            'src/node_modules/fsevents/node_modules/fstream-ignore/*',
            'src/node_modules/fsevents/node_modules/fstream/*',
            'src/node_modules/fsevents/node_modules/fstream/examples/*',
            'src/node_modules/fsevents/node_modules/fstream/lib/*',
            'src/node_modules/fsevents/node_modules/gauge/*',
            'src/node_modules/fsevents/node_modules/getpass/*',
            'src/node_modules/fsevents/node_modules/getpass/lib/*',
            'src/node_modules/fsevents/node_modules/getpass/node_modules/*',
            'src/node_modules/fsevents/node_modules/getpass/node_modules/assert-plus/*',
            'src/node_modules/fsevents/node_modules/glob/*',
            'src/node_modules/fsevents/node_modules/graceful-fs/*',
            'src/node_modules/fsevents/node_modules/har-schema/*',
            'src/node_modules/fsevents/node_modules/har-schema/lib/*',
            'src/node_modules/fsevents/node_modules/har-validator/*',
            'src/node_modules/fsevents/node_modules/har-validator/lib/*',
            'src/node_modules/fsevents/node_modules/har-validator/lib/browser/*',
            'src/node_modules/fsevents/node_modules/har-validator/lib/node4/*',
            'src/node_modules/fsevents/node_modules/har-validator/lib/node6/*',
            'src/node_modules/fsevents/node_modules/har-validator/lib/node7/*',
            'src/node_modules/fsevents/node_modules/har-validator/src/*',
            'src/node_modules/fsevents/node_modules/has-unicode/*',
            'src/node_modules/fsevents/node_modules/hawk/*',
            'src/node_modules/fsevents/node_modules/hawk/dist/*',
            'src/node_modules/fsevents/node_modules/hawk/example/*',
            'src/node_modules/fsevents/node_modules/hawk/images/*',
            'src/node_modules/fsevents/node_modules/hawk/lib/*',
            'src/node_modules/fsevents/node_modules/hawk/test/*',
            'src/node_modules/fsevents/node_modules/hoek/*',
            'src/node_modules/fsevents/node_modules/hoek/images/*',
            'src/node_modules/fsevents/node_modules/hoek/lib/*',
            'src/node_modules/fsevents/node_modules/hoek/test/*',
            'src/node_modules/fsevents/node_modules/hoek/test/modules/*',
            'src/node_modules/fsevents/node_modules/http-signature/*',
            'src/node_modules/fsevents/node_modules/http-signature/lib/*',
            'src/node_modules/fsevents/node_modules/inflight/*',
            'src/node_modules/fsevents/node_modules/inherits/*',
            'src/node_modules/fsevents/node_modules/ini/*',
            'src/node_modules/fsevents/node_modules/is-fullwidth-code-point/*',
            'src/node_modules/fsevents/node_modules/is-typedarray/*',
            'src/node_modules/fsevents/node_modules/isarray/*',
            'src/node_modules/fsevents/node_modules/isstream/*',
            'src/node_modules/fsevents/node_modules/jodid25519/*',
            'src/node_modules/fsevents/node_modules/jodid25519/lib/*',
            'src/node_modules/fsevents/node_modules/jsbn/*',
            'src/node_modules/fsevents/node_modules/json-schema/*',
            'src/node_modules/fsevents/node_modules/json-schema/draft-00/*',
            'src/node_modules/fsevents/node_modules/json-schema/draft-01/*',
            'src/node_modules/fsevents/node_modules/json-schema/draft-02/*',
            'src/node_modules/fsevents/node_modules/json-schema/draft-03/*',
            'src/node_modules/fsevents/node_modules/json-schema/draft-03/examples/*',
            'src/node_modules/fsevents/node_modules/json-schema/draft-04/*',
            'src/node_modules/fsevents/node_modules/json-schema/lib/*',
            'src/node_modules/fsevents/node_modules/json-schema/test/*',
            'src/node_modules/fsevents/node_modules/json-stable-stringify/*',
            'src/node_modules/fsevents/node_modules/json-stable-stringify/example/*',
            'src/node_modules/fsevents/node_modules/json-stable-stringify/test/*',
            'src/node_modules/fsevents/node_modules/json-stringify-safe/*',
            'src/node_modules/fsevents/node_modules/json-stringify-safe/test/*',
            'src/node_modules/fsevents/node_modules/jsonify/*',
            'src/node_modules/fsevents/node_modules/jsonify/lib/*',
            'src/node_modules/fsevents/node_modules/jsonify/test/*',
            'src/node_modules/fsevents/node_modules/jsprim/*',
            'src/node_modules/fsevents/node_modules/jsprim/lib/*',
            'src/node_modules/fsevents/node_modules/jsprim/node_modules/*',
            'src/node_modules/fsevents/node_modules/jsprim/node_modules/assert-plus/*',
            'src/node_modules/fsevents/node_modules/mime-db/*',
            'src/node_modules/fsevents/node_modules/mime-types/*',
            'src/node_modules/fsevents/node_modules/minimatch/*',
            'src/node_modules/fsevents/node_modules/minimist/*',
            'src/node_modules/fsevents/node_modules/minimist/example/*',
            'src/node_modules/fsevents/node_modules/minimist/test/*',
            'src/node_modules/fsevents/node_modules/mkdirp/*',
            'src/node_modules/fsevents/node_modules/mkdirp/bin/*',
            'src/node_modules/fsevents/node_modules/mkdirp/examples/*',
            'src/node_modules/fsevents/node_modules/mkdirp/test/*',
            'src/node_modules/fsevents/node_modules/ms/*',
            'src/node_modules/fsevents/node_modules/node-pre-gyp/*',
            'src/node_modules/fsevents/node_modules/node-pre-gyp/bin/*',
            'src/node_modules/fsevents/node_modules/node-pre-gyp/lib/*',
            'src/node_modules/fsevents/node_modules/node-pre-gyp/lib/util/*',
            'src/node_modules/fsevents/node_modules/node-pre-gyp/lib/util/nw-pre-gyp/*',
            'src/node_modules/fsevents/node_modules/nopt/*',
            'src/node_modules/fsevents/node_modules/nopt/bin/*',
            'src/node_modules/fsevents/node_modules/nopt/examples/*',
            'src/node_modules/fsevents/node_modules/nopt/lib/*',
            'src/node_modules/fsevents/node_modules/nopt/test/*',
            'src/node_modules/fsevents/node_modules/npmlog/*',
            'src/node_modules/fsevents/node_modules/number-is-nan/*',
            'src/node_modules/fsevents/node_modules/oauth-sign/*',
            'src/node_modules/fsevents/node_modules/object-assign/*',
            'src/node_modules/fsevents/node_modules/once/*',
            'src/node_modules/fsevents/node_modules/os-homedir/*',
            'src/node_modules/fsevents/node_modules/os-tmpdir/*',
            'src/node_modules/fsevents/node_modules/osenv/*',
            'src/node_modules/fsevents/node_modules/osenv/test/*',
            'src/node_modules/fsevents/node_modules/path-is-absolute/*',
            'src/node_modules/fsevents/node_modules/performance-now/*',
            'src/node_modules/fsevents/node_modules/performance-now/lib/*',
            'src/node_modules/fsevents/node_modules/performance-now/src/*',
            'src/node_modules/fsevents/node_modules/performance-now/test/*',
            'src/node_modules/fsevents/node_modules/process-nextick-args/*',
            'src/node_modules/fsevents/node_modules/punycode/*',
            'src/node_modules/fsevents/node_modules/qs/*',
            'src/node_modules/fsevents/node_modules/qs/dist/*',
            'src/node_modules/fsevents/node_modules/qs/lib/*',
            'src/node_modules/fsevents/node_modules/qs/test/*',
            'src/node_modules/fsevents/node_modules/rc/*',
            'src/node_modules/fsevents/node_modules/rc/lib/*',
            'src/node_modules/fsevents/node_modules/rc/node_modules/*',
            'src/node_modules/fsevents/node_modules/rc/node_modules/minimist/*',
            'src/node_modules/fsevents/node_modules/rc/node_modules/minimist/example/*',
            'src/node_modules/fsevents/node_modules/rc/node_modules/minimist/test/*',
            'src/node_modules/fsevents/node_modules/rc/test/*',
            'src/node_modules/fsevents/node_modules/readable-stream/*',
            'src/node_modules/fsevents/node_modules/readable-stream/doc/*',
            'src/node_modules/fsevents/node_modules/readable-stream/doc/wg-meetings/*',
            'src/node_modules/fsevents/node_modules/readable-stream/lib/*',
            'src/node_modules/fsevents/node_modules/readable-stream/lib/internal/*',
            'src/node_modules/fsevents/node_modules/readable-stream/lib/internal/streams/*',
            'src/node_modules/fsevents/node_modules/request/*',
            'src/node_modules/fsevents/node_modules/request/lib/*',
            'src/node_modules/fsevents/node_modules/rimraf/*',
            'src/node_modules/fsevents/node_modules/safe-buffer/*',
            'src/node_modules/fsevents/node_modules/semver/*',
            'src/node_modules/fsevents/node_modules/semver/bin/*',
            'src/node_modules/fsevents/node_modules/set-blocking/*',
            'src/node_modules/fsevents/node_modules/signal-exit/*',
            'src/node_modules/fsevents/node_modules/sntp/*',
            'src/node_modules/fsevents/node_modules/sntp/examples/*',
            'src/node_modules/fsevents/node_modules/sntp/lib/*',
            'src/node_modules/fsevents/node_modules/sntp/test/*',
            'src/node_modules/fsevents/node_modules/sshpk/*',
            'src/node_modules/fsevents/node_modules/sshpk/bin/*',
            'src/node_modules/fsevents/node_modules/sshpk/lib/*',
            'src/node_modules/fsevents/node_modules/sshpk/lib/formats/*',
            'src/node_modules/fsevents/node_modules/sshpk/man/*',
            'src/node_modules/fsevents/node_modules/sshpk/man/man1/*',
            'src/node_modules/fsevents/node_modules/sshpk/node_modules/*',
            'src/node_modules/fsevents/node_modules/sshpk/node_modules/assert-plus/*',
            'src/node_modules/fsevents/node_modules/string-width/*',
            'src/node_modules/fsevents/node_modules/string_decoder/*',
            'src/node_modules/fsevents/node_modules/string_decoder/lib/*',
            'src/node_modules/fsevents/node_modules/stringstream/*',
            'src/node_modules/fsevents/node_modules/strip-ansi/*',
            'src/node_modules/fsevents/node_modules/strip-json-comments/*',
            'src/node_modules/fsevents/node_modules/tar-pack/*',
            'src/node_modules/fsevents/node_modules/tar-pack/test/*',
            'src/node_modules/fsevents/node_modules/tar-pack/test/fixtures/*',
            'src/node_modules/fsevents/node_modules/tar-pack/test/fixtures/to-pack/*',
            'src/node_modules/fsevents/node_modules/tar/*',
            'src/node_modules/fsevents/node_modules/tar/examples/*',
            'src/node_modules/fsevents/node_modules/tar/lib/*',
            'src/node_modules/fsevents/node_modules/tar/test/*',
            'src/node_modules/fsevents/node_modules/tough-cookie/*',
            'src/node_modules/fsevents/node_modules/tough-cookie/lib/*',
            'src/node_modules/fsevents/node_modules/tunnel-agent/*',
            'src/node_modules/fsevents/node_modules/tweetnacl/*',
            'src/node_modules/fsevents/node_modules/uid-number/*',
            'src/node_modules/fsevents/node_modules/util-deprecate/*',
            'src/node_modules/fsevents/node_modules/uuid/*',
            'src/node_modules/fsevents/node_modules/uuid/bin/*',
            'src/node_modules/fsevents/node_modules/uuid/lib/*',
            'src/node_modules/fsevents/node_modules/uuid/test/*',
            'src/node_modules/fsevents/node_modules/verror/*',
            'src/node_modules/fsevents/node_modules/verror/examples/*',
            'src/node_modules/fsevents/node_modules/verror/lib/*',
            'src/node_modules/fsevents/node_modules/verror/tests/*',
            'src/node_modules/fsevents/node_modules/wide-align/*',
            'src/node_modules/fsevents/node_modules/wrappy/*',
            'src/node_modules/fsevents/src/*',
            'src/node_modules/function.name/*',
            'src/node_modules/function.name/dist/*',
            'src/node_modules/function.name/lib/*',
            'src/node_modules/git-package-json/*',
            'src/node_modules/git-package-json/lib/*',
            'src/node_modules/git-source/*',
            'src/node_modules/git-source/lib/*',
            'src/node_modules/git-up/*',
            'src/node_modules/git-up/example/*',
            'src/node_modules/git-up/lib/*',
            'src/node_modules/git-up/test/*',
            'src/node_modules/git-url-parse/*',
            'src/node_modules/git-url-parse/example/*',
            'src/node_modules/git-url-parse/lib/*',
            'src/node_modules/git-url-parse/test/*',
            'src/node_modules/glob-base/*',
            'src/node_modules/glob-parent/*',
            'src/node_modules/glob/*',
            'src/node_modules/globals/*',
            'src/node_modules/got/*',
            'src/node_modules/graceful-fs/*',
            'src/node_modules/gry/*',
            'src/node_modules/gry/lib/*',
            'src/node_modules/has-ansi/*',
            'src/node_modules/home-or-tmp/*',
            'src/node_modules/hosted-git-info/*',
            'src/node_modules/inflight/*',
            'src/node_modules/inherits/*',
            'src/node_modules/ini/*',
            'src/node_modules/invariant/*',
            'src/node_modules/is-arrayish/*',
            'src/node_modules/is-binary-path/*',
            'src/node_modules/is-buffer/*',
            'src/node_modules/is-buffer/test/*',
            'src/node_modules/is-builtin-module/*',
            'src/node_modules/is-dotfile/*',
            'src/node_modules/is-equal-shallow/*',
            'src/node_modules/is-extendable/*',
            'src/node_modules/is-extglob/*',
            'src/node_modules/is-finite/*',
            'src/node_modules/is-glob/*',
            'src/node_modules/is-number/*',
            'src/node_modules/is-posix-bracket/*',
            'src/node_modules/is-primitive/*',
            'src/node_modules/is-redirect/*',
            'src/node_modules/is-retry-allowed/*',
            'src/node_modules/is-ssh/*',
            'src/node_modules/is-ssh/example/*',
            'src/node_modules/is-ssh/lib/*',
            'src/node_modules/is-ssh/test/*',
            'src/node_modules/is-stream/*',
            'src/node_modules/isarray/*',
            'src/node_modules/isobject/*',
            'src/node_modules/iterate-object/*',
            'src/node_modules/iterate-object/example/*',
            'src/node_modules/iterate-object/lib/*',
            'src/node_modules/js-tokens/*',
            'src/node_modules/jsesc/*',
            'src/node_modules/jsesc/bin/*',
            'src/node_modules/jsesc/man/*',
            'src/node_modules/json5/*',
            'src/node_modules/json5/lib/*',
            'src/node_modules/kind-of/*',
            'src/node_modules/limit-it/*',
            'src/node_modules/limit-it/lib/*',
            'src/node_modules/lodash/*',
            'src/node_modules/lodash/fp/*',
            'src/node_modules/loose-envify/*',
            'src/node_modules/lowercase-keys/*',
            'src/node_modules/micromatch/*',
            'src/node_modules/micromatch/lib/*',
            'src/node_modules/minimatch/*',
            'src/node_modules/minimist/*',
            'src/node_modules/minimist/example/*',
            'src/node_modules/minimist/test/*',
            'src/node_modules/mkdirp/*',
            'src/node_modules/mkdirp/bin/*',
            'src/node_modules/mkdirp/examples/*',
            'src/node_modules/mkdirp/node_modules/*',
            'src/node_modules/mkdirp/node_modules/minimist/*',
            'src/node_modules/mkdirp/node_modules/minimist/example/*',
            'src/node_modules/mkdirp/node_modules/minimist/test/*',
            'src/node_modules/mkdirp/test/*',
            'src/node_modules/ms/*',
            'src/node_modules/nan/*',
            'src/node_modules/nan/doc/*',
            'src/node_modules/nan/tools/*',
            'src/node_modules/node-status-codes/*',
            'src/node_modules/noop6/*',
            'src/node_modules/noop6/lib/*',
            'src/node_modules/normalize-package-data/*',
            'src/node_modules/normalize-package-data/lib/*',
            'src/node_modules/normalize-path/*',
            'src/node_modules/number-is-nan/*',
            'src/node_modules/oargv/*',
            'src/node_modules/oargv/lib/*',
            'src/node_modules/obj-def/*',
            'src/node_modules/obj-def/lib/*',
            'src/node_modules/object-assign/*',
            'src/node_modules/object.omit/*',
            'src/node_modules/once/*',
            'src/node_modules/one-by-one/*',
            'src/node_modules/one-by-one/lib/*',
            'src/node_modules/os-homedir/*',
            'src/node_modules/os-tmpdir/*',
            'src/node_modules/output-file-sync/*',
            'src/node_modules/package-json-path/*',
            'src/node_modules/package-json-path/lib/*',
            'src/node_modules/package-json/*',
            'src/node_modules/package.json/*',
            'src/node_modules/package.json/lib/*',
            'src/node_modules/parse-glob/*',
            'src/node_modules/parse-json/*',
            'src/node_modules/parse-json/vendor/*',
            'src/node_modules/parse-url/*',
            'src/node_modules/parse-url/lib/*',
            'src/node_modules/path-is-absolute/*',
            'src/node_modules/pinkie-promise/*',
            'src/node_modules/pinkie/*',
            'src/node_modules/prepend-http/*',
            'src/node_modules/preserve/*',
            'src/node_modules/private/*',
            'src/node_modules/process-nextick-args/*',
            'src/node_modules/protocols/*',
            'src/node_modules/protocols/lib/*',
            'src/node_modules/r-json/*',
            'src/node_modules/r-json/lib/*',
            'src/node_modules/r-package-json/*',
            'src/node_modules/r-package-json/lib/*',
            'src/node_modules/randomatic/*',
            'src/node_modules/randomatic/node_modules/*',
            'src/node_modules/randomatic/node_modules/is-number/*',
            'src/node_modules/randomatic/node_modules/is-number/node_modules/*',
            'src/node_modules/randomatic/node_modules/is-number/node_modules/kind-of/*',
            'src/node_modules/randomatic/node_modules/kind-of/*',
            'src/node_modules/rc/*',
            'src/node_modules/rc/lib/*',
            'src/node_modules/rc/test/*',
            'src/node_modules/read-all-stream/*',
            'src/node_modules/readable-stream/*',
            'src/node_modules/readable-stream/doc/*',
            'src/node_modules/readable-stream/doc/wg-meetings/*',
            'src/node_modules/readable-stream/lib/*',
            'src/node_modules/readable-stream/lib/internal/*',
            'src/node_modules/readable-stream/lib/internal/streams/*',
            'src/node_modules/readdirp/*',
            'src/node_modules/readdirp/examples/*',
            'src/node_modules/readdirp/test/*',
            'src/node_modules/readdirp/test/bed/*',
            'src/node_modules/readdirp/test/bed/root_dir1/*',
            'src/node_modules/readdirp/test/bed/root_dir1/root_dir1_subdir1/*',
            'src/node_modules/readdirp/test/bed/root_dir2/*',
            'src/node_modules/regenerate/*',
            'src/node_modules/regenerator-runtime/*',
            'src/node_modules/regenerator-transform/*',
            'src/node_modules/regenerator-transform/lib/*',
            'src/node_modules/regenerator-transform/src/*',
            'src/node_modules/regex-cache/*',
            'src/node_modules/regexpu-core/*',
            'src/node_modules/regexpu-core/data/*',
            'src/node_modules/registry-auth-token/*',
            'src/node_modules/registry-auth-token/test/*',
            'src/node_modules/registry-url/*',
            'src/node_modules/regjsgen/*',
            'src/node_modules/regjsparser/*',
            'src/node_modules/regjsparser/bin/*',
            'src/node_modules/regjsparser/node_modules/*',
            'src/node_modules/regjsparser/node_modules/.bin/*',
            'src/node_modules/regjsparser/node_modules/jsesc/*',
            'src/node_modules/regjsparser/node_modules/jsesc/bin/*',
            'src/node_modules/regjsparser/node_modules/jsesc/man/*',
            'src/node_modules/remove-trailing-separator/*',
            'src/node_modules/repeat-element/*',
            'src/node_modules/repeat-string/*',
            'src/node_modules/repeating/*',
            'src/node_modules/safe-buffer/*',
            'src/node_modules/semver/*',
            'src/node_modules/semver/bin/*',
            'src/node_modules/set-immediate-shim/*',
            'src/node_modules/slash/*',
            'src/node_modules/sliced/*',
            'src/node_modules/source-map-support/*',
            'src/node_modules/source-map/*',
            'src/node_modules/source-map/dist/*',
            'src/node_modules/source-map/lib/*',
            'src/node_modules/spdx-correct/*',
            'src/node_modules/spdx-expression-parse/*',
            'src/node_modules/spdx-license-ids/*',
            'src/node_modules/string_decoder/*',
            'src/node_modules/string_decoder/lib/*',
            'src/node_modules/strip-ansi/*',
            'src/node_modules/strip-json-comments/*',
            'src/node_modules/supports-color/*',
            'src/node_modules/timed-out/*',
            'src/node_modules/tmp/*',
            'src/node_modules/tmp/lib/*',
            'src/node_modules/tmp/test/*',
            'src/node_modules/tmp/test/symlinkme/*',
            'src/node_modules/to-fast-properties/*',
            'src/node_modules/trim-right/*',
            'src/node_modules/typpy/*',
            'src/node_modules/typpy/lib/*',
            'src/node_modules/ul/*',
            'src/node_modules/ul/lib/*',
            'src/node_modules/unzip-response/*',
            'src/node_modules/url-parse-lax/*',
            'src/node_modules/user-home/*',
            'src/node_modules/util-deprecate/*',
            'src/node_modules/v8flags/*',
            'src/node_modules/validate-npm-package-license/*',
            'src/node_modules/wrappy/*',
            'templates/*']}

install_requires = \
['astor',
 'click',
 'eventlet',
 'flask',
 'flask-socketio',
 'future',
 'jinja2',
 'markdown',
 'msgpack-python']

setup(name='bowtie',
      version='0.7.0',
      description='Interactive dashboard toolkit.',
      author='Jacques Kvam',
      author_email='jwkvam+pypi@gmail.com',
      url='https://github.com/jwkvam/bowtie',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Compile tests."""

from bowtie import App
from bowtie.control import Nouislider
from bowtie.visual import Plotly
from bowtie.tests.utils import reset_uuid


def callback(*args):
    """dummy function"""
    # pylint: disable=unused-argument
    pass


# pylint: disable=unused-argument
def test_build(build_path, monkeypatch):
    """Tests the build process."""
    monkeypatch.setattr(App, '_sourcefile', lambda self: 'bowtie.tests.test_compile')
    reset_uuid()
    ctrl = Nouislider()
    viz = Plotly()

    app = App()
    app.add_sidebar(ctrl)
    app.add(viz)
    app.subscribe(callback, ctrl.on_change)
    # pylint: disable=protected-access
    app._build()
